# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

import typing

lib: typing.Any
ffi: typing.Any



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from cryptography.hazmat.primitives.asymmetric import mldsa
from cryptography.utils import Buffer

class MLDSA44PrivateKey: ...
class MLDSA44PublicKey: ...
class MLDSA65PrivateKey: ...
class MLDSA65PublicKey: ...
class MLDSA87PrivateKey: ...
class MLDSA87PublicKey: ...

def generate_mldsa44_key() -> mldsa.MLDSA44PrivateKey: ...
def from_mldsa44_public_bytes(data: bytes) -> mldsa.MLDSA44PublicKey: ...
def from_mldsa44_seed_bytes(data: Buffer) -> mldsa.MLDSA44PrivateKey: ...
def generate_mldsa65_key() -> mldsa.MLDSA65PrivateKey: ...
def from_mldsa65_public_bytes(data: bytes) -> mldsa.MLDSA65PublicKey: ...
def from_mldsa65_seed_bytes(data: Buffer) -> mldsa.MLDSA65PrivateKey: ...
def generate_mldsa87_key() -> mldsa.MLDSA87PrivateKey: ...
def from_mldsa87_public_bytes(data: bytes) -> mldsa.MLDSA87PublicKey: ...
def from_mldsa87_seed_bytes(data: Buffer) -> mldsa.MLDSA87PrivateKey: ...



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from __future__ import annotations

from cryptography import utils
from cryptography.hazmat.bindings._rust import openssl as rust_openssl
from cryptography.hazmat.primitives.kdf import KeyDerivationFunction


class Mode(utils.Enum):
    CounterMode = "ctr"


class CounterLocation(utils.Enum):
    BeforeFixed = "before_fixed"
    AfterFixed = "after_fixed"
    MiddleFixed = "middle_fixed"


KBKDFHMAC = rust_openssl.kdf.KBKDFHMAC
KeyDerivationFunction.register(KBKDFHMAC)

KBKDFCMAC = rust_openssl.kdf.KBKDFCMAC
KeyDerivationFunction.register(KBKDFCMAC)



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from cryptography import x509
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.serialization import pkcs7
from cryptography.utils import Buffer

class TestCertificate:
    not_after_tag: int
    not_before_tag: int
    issuer_value_tags: list[int]
    subject_value_tags: list[int]

def test_parse_certificate(data: bytes) -> TestCertificate: ...
def pkcs7_verify(
    encoding: serialization.Encoding,
    sig: bytes,
    msg: Buffer | None,
    certs: list[x509.Certificate],
    options: list[pkcs7.PKCS7Options],
) -> None: ...



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from __future__ import annotations

from cryptography.hazmat.bindings._rust import openssl as rust_openssl
from cryptography.hazmat.primitives.kdf import KeyDerivationFunction

ConcatKDFHash = rust_openssl.kdf.ConcatKDFHash
ConcatKDFHMAC = rust_openssl.kdf.ConcatKDFHMAC

KeyDerivationFunction.register(ConcatKDFHash)
KeyDerivationFunction.register(ConcatKDFHMAC)

__all__ = ["ConcatKDFHMAC", "ConcatKDFHash"]



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from __future__ import annotations

from cryptography.hazmat.bindings._rust import openssl as rust_openssl

AEAD = rust_openssl.hpke.AEAD
KDF = rust_openssl.hpke.KDF
KEM = rust_openssl.hpke.KEM
MLKEM768X25519PrivateKey = rust_openssl.hpke.MLKEM768X25519PrivateKey
MLKEM768X25519PublicKey = rust_openssl.hpke.MLKEM768X25519PublicKey
MLKEM1024P384PrivateKey = rust_openssl.hpke.MLKEM1024P384PrivateKey
MLKEM1024P384PublicKey = rust_openssl.hpke.MLKEM1024P384PublicKey
Suite = rust_openssl.hpke.Suite

__all__ = [
    "AEAD",
    "KDF",
    "KEM",
    "MLKEM768X25519PrivateKey",
    "MLKEM768X25519PublicKey",
    "MLKEM1024P384PrivateKey",
    "MLKEM1024P384PublicKey",
    "Suite",
]



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from __future__ import annotations

import abc

from cryptography.exceptions import UnsupportedAlgorithm, _Reasons
from cryptography.hazmat.bindings._rust import openssl as rust_openssl
from cryptography.hazmat.primitives import _serialization
from cryptography.utils import Buffer


class MLKEM768PublicKey(metaclass=abc.ABCMeta):
    @classmethod
    def from_public_bytes(cls, data: Buffer) -> MLKEM768PublicKey:
        from cryptography.hazmat.backends.openssl.backend import backend

        if not backend.mlkem_supported():
            raise UnsupportedAlgorithm(
                "ML-KEM-768 is not supported by this backend.",
                _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM,
            )

        return rust_openssl.mlkem.from_mlkem768_public_bytes(data)

    @abc.abstractmethod
    def encapsulate(self) -> tuple[bytes, bytes]:
        """
        Encapsulate: returns (shared_secret, ciphertext).
        """

    @abc.abstractmethod
    def public_bytes(
        self,
        encoding: _serialization.Encoding,
        format: _serialization.PublicFormat,
    ) -> bytes:
        """
        The serialized bytes of the public key.
        """

    @abc.abstractmethod
    def public_bytes_raw(self) -> bytes:
        """
        The raw bytes of the public key.
        Equivalent to public_bytes(Raw, Raw).

        The public key is 1,184 bytes for ML-KEM-768.
        """

    @abc.abstractmethod
    def __eq__(self, other: object) -> bool:
        """
        Checks equality.
        """

    @abc.abstractmethod
    def __copy__(self) -> MLKEM768PublicKey:
        """
        Returns a copy.
        """

    @abc.abstractmethod
    def __deepcopy__(self, memo: dict) -> MLKEM768PublicKey:
        """
        Returns a deep copy.
        """


if hasattr(rust_openssl, "mlkem"):
    MLKEM768PublicKey.register(rust_openssl.mlkem.MLKEM768PublicKey)


class MLKEM768PrivateKey(metaclass=abc.ABCMeta):
    @classmethod
    def generate(cls) -> MLKEM768PrivateKey:
        from cryptography.hazmat.backends.openssl.backend import backend

        if not backend.mlkem_supported():
            raise UnsupportedAlgorithm(
                "ML-KEM-768 is not supported by this backend.",
                _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM,
            )

        return rust_openssl.mlkem.generate_mlkem768_key()

    @classmethod
    def from_seed_bytes(cls, data: Buffer) -> MLKEM768PrivateKey:
        from cryptography.hazmat.backends.openssl.backend import backend

        if not backend.mlkem_supported():
            raise UnsupportedAlgorithm(
                "ML-KEM-768 is not supported by this backend.",
                _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM,
            )

        return rust_openssl.mlkem.from_mlkem768_seed_bytes(data)

    @abc.abstractmethod
    def decapsulate(self, ciphertext: Buffer) -> bytes:
        """
        Decapsulate: returns shared_secret.
        """

    @abc.abstractmethod
    def public_key(self) -> MLKEM768PublicKey:
        """
        The MLKEM768PublicKey derived from this private key.
        """

    @abc.abstractmethod
    def private_bytes(
        self,
        encoding: _serialization.Encoding,
        format: _serialization.PrivateFormat,
        encryption_algorithm: _serialization.KeySerializationEncryption,
    ) -> bytes:
        """
        The serialized bytes of the private key.
        """

    @abc.abstractmethod
    def private_bytes_raw(self) -> bytes:
        """
        The raw bytes of the private key (64-byte seed).
        Equivalent to private_bytes(Raw, Raw, NoEncryption()).
        """

    @abc.abstractmethod
    def __copy__(self) -> MLKEM768PrivateKey:
        """
        Returns a copy.
        """

    @abc.abstractmethod
    def __deepcopy__(self, memo: dict) -> MLKEM768PrivateKey:
        """
        Returns a deep copy.
        """


if hasattr(rust_openssl, "mlkem"):
    MLKEM768PrivateKey.register(rust_openssl.mlkem.MLKEM768PrivateKey)


class MLKEM1024PublicKey(metaclass=abc.ABCMeta):
    @classmethod
    def from_public_bytes(cls, data: Buffer) -> MLKEM1024PublicKey:
        from cryptography.hazmat.backends.openssl.backend import backend

        if not backend.mlkem_supported():
            raise UnsupportedAlgorithm(
                "ML-KEM-1024 is not supported by this backend.",
                _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM,
            )

        return rust_openssl.mlkem.from_mlkem1024_public_bytes(data)

    @abc.abstractmethod
    def encapsulate(self) -> tuple[bytes, bytes]:
        """
        Encapsulate: returns (shared_secret, ciphertext).
        """

    @abc.abstractmethod
    def public_bytes(
        self,
        encoding: _serialization.Encoding,
        format: _serialization.PublicFormat,
    ) -> bytes:
        """
        The serialized bytes of the public key.
        """

    @abc.abstractmethod
    def public_bytes_raw(self) -> bytes:
        """
        The raw bytes of the public key.
        Equivalent to public_bytes(Raw, Raw).

        The public key is 1,568 bytes for ML-KEM-1024.
        """

    @abc.abstractmethod
    def __eq__(self, other: object) -> bool:
        """
        Checks equality.
        """

    @abc.abstractmethod
    def __copy__(self) -> MLKEM1024PublicKey:
        """
        Returns a copy.
        """

    @abc.abstractmethod
    def __deepcopy__(self, memo: dict) -> MLKEM1024PublicKey:
        """
        Returns a deep copy.
        """


if hasattr(rust_openssl, "mlkem"):
    MLKEM1024PublicKey.register(rust_openssl.mlkem.MLKEM1024PublicKey)


class MLKEM1024PrivateKey(metaclass=abc.ABCMeta):
    @classmethod
    def generate(cls) -> MLKEM1024PrivateKey:
        from cryptography.hazmat.backends.openssl.backend import backend

        if not backend.mlkem_supported():
            raise UnsupportedAlgorithm(
                "ML-KEM-1024 is not supported by this backend.",
                _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM,
            )

        return rust_openssl.mlkem.generate_mlkem1024_key()

    @classmethod
    def from_seed_bytes(cls, data: Buffer) -> MLKEM1024PrivateKey:
        from cryptography.hazmat.backends.openssl.backend import backend

        if not backend.mlkem_supported():
            raise UnsupportedAlgorithm(
                "ML-KEM-1024 is not supported by this backend.",
                _Reasons.UNSUPPORTED_PUBLIC_KEY_ALGORITHM,
            )

        return rust_openssl.mlkem.from_mlkem1024_seed_bytes(data)

    @abc.abstractmethod
    def decapsulate(self, ciphertext: Buffer) -> bytes:
        """
        Decapsulate: returns shared_secret.
        """

    @abc.abstractmethod
    def public_key(self) -> MLKEM1024PublicKey:
        """
        The MLKEM1024PublicKey derived from this private key.
        """

    @abc.abstractmethod
    def private_bytes(
        self,
        encoding: _serialization.Encoding,
        format: _serialization.PrivateFormat,
        encryption_algorithm: _serialization.KeySerializationEncryption,
    ) -> bytes:
        """
        The serialized bytes of the private key.
        """

    @abc.abstractmethod
    def private_bytes_raw(self) -> bytes:
        """
        The raw bytes of the private key (64-byte seed).
        Equivalent to private_bytes(Raw, Raw, NoEncryption()).
        """

    @abc.abstractmethod
    def __copy__(self) -> MLKEM1024PrivateKey:
        """
        Returns a copy.
        """

    @abc.abstractmethod
    def __deepcopy__(self, memo: dict) -> MLKEM1024PrivateKey:
        """
        Returns a deep copy.
        """


if hasattr(rust_openssl, "mlkem"):
    MLKEM1024PrivateKey.register(rust_openssl.mlkem.MLKEM1024PrivateKey)



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from __future__ import annotations

import abc

from cryptography.hazmat.bindings._rust import openssl as rust_openssl
from cryptography.hazmat.primitives import _serialization

generate_parameters = rust_openssl.dh.generate_parameters


DHPrivateNumbers = rust_openssl.dh.DHPrivateNumbers
DHPublicNumbers = rust_openssl.dh.DHPublicNumbers
DHParameterNumbers = rust_openssl.dh.DHParameterNumbers


class DHParameters(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def generate_private_key(self) -> DHPrivateKey:
        """
        Generates and returns a DHPrivateKey.
        """

    @abc.abstractmethod
    def parameter_bytes(
        self,
        encoding: _serialization.Encoding,
        format: _serialization.ParameterFormat,
    ) -> bytes:
        """
        Returns the parameters serialized as bytes.
        """

    @abc.abstractmethod
    def parameter_numbers(self) -> DHParameterNumbers:
        """
        Returns a DHParameterNumbers.
        """


DHParametersWithSerialization = DHParameters
DHParameters.register(rust_openssl.dh.DHParameters)


class DHPublicKey(metaclass=abc.ABCMeta):
    @property
    @abc.abstractmethod
    def key_size(self) -> int:
        """
        The bit length of the prime modulus.
        """

    @abc.abstractmethod
    def parameters(self) -> DHParameters:
        """
        The DHParameters object associated with this public key.
        """

    @abc.abstractmethod
    def public_numbers(self) -> DHPublicNumbers:
        """
        Returns a DHPublicNumbers.
        """

    @abc.abstractmethod
    def public_bytes(
        self,
        encoding: _serialization.Encoding,
        format: _serialization.PublicFormat,
    ) -> bytes:
        """
        Returns the key serialized as bytes.
        """

    @abc.abstractmethod
    def __eq__(self, other: object) -> bool:
        """
        Checks equality.
        """

    @abc.abstractmethod
    def __copy__(self) -> DHPublicKey:
        """
        Returns a copy.
        """

    @abc.abstractmethod
    def __deepcopy__(self, memo: dict) -> DHPublicKey:
        """
        Returns a deep copy.
        """


DHPublicKeyWithSerialization = DHPublicKey
DHPublicKey.register(rust_openssl.dh.DHPublicKey)


class DHPrivateKey(metaclass=abc.ABCMeta):
    @property
    @abc.abstractmethod
    def key_size(self) -> int:
        """
        The bit length of the prime modulus.
        """

    @abc.abstractmethod
    def public_key(self) -> DHPublicKey:
        """
        The DHPublicKey associated with this private key.
        """

    @abc.abstractmethod
    def parameters(self) -> DHParameters:
        """
        The DHParameters object associated with this private key.
        """

    @abc.abstractmethod
    def exchange(self, peer_public_key: DHPublicKey) -> bytes:
        """
        Given peer's DHPublicKey, carry out the key exchange and
        return shared key as bytes.
        """

    @abc.abstractmethod
    def private_numbers(self) -> DHPrivateNumbers:
        """
        Returns a DHPrivateNumbers.
        """

    @abc.abstractmethod
    def private_bytes(
        self,
        encoding: _serialization.Encoding,
        format: _serialization.PrivateFormat,
        encryption_algorithm: _serialization.KeySerializationEncryption,
    ) -> bytes:
        """
        Returns the key serialized as bytes.
        """

    @abc.abstractmethod
    def __copy__(self) -> DHPrivateKey:
        """
        Returns a copy.
        """

    @abc.abstractmethod
    def __deepcopy__(self, memo: dict) -> DHPrivateKey:
        """
        Returns a deep copy.
        """


DHPrivateKeyWithSerialization = DHPrivateKey
DHPrivateKey.register(rust_openssl.dh.DHPrivateKey)



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

import typing

from cryptography.hazmat.primitives.hashes import HashAlgorithm
from cryptography.hazmat.primitives.kdf.kbkdf import CounterLocation, Mode
from cryptography.utils import Buffer

class PBKDF2HMAC:
    def __init__(
        self,
        algorithm: HashAlgorithm,
        length: int,
        salt: bytes,
        iterations: int,
        backend: typing.Any = None,
    ) -> None: ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class Scrypt:
    def __init__(
        self,
        salt: bytes,
        length: int,
        n: int,
        r: int,
        p: int,
        backend: typing.Any = None,
    ) -> None: ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class Argon2d:
    def __init__(
        self,
        *,
        salt: bytes,
        length: int,
        iterations: int,
        lanes: int,
        memory_cost: int,
        ad: bytes | None = None,
        secret: bytes | None = None,
    ) -> None: ...
    def derive(self, key_material: bytes) -> bytes: ...
    def derive_into(self, key_material: bytes, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...
    def derive_phc_encoded(self, key_material: bytes) -> str: ...
    @classmethod
    def verify_phc_encoded(
        cls, key_material: bytes, phc_encoded: str, secret: bytes | None = None
    ) -> None: ...

class Argon2i:
    def __init__(
        self,
        *,
        salt: bytes,
        length: int,
        iterations: int,
        lanes: int,
        memory_cost: int,
        ad: bytes | None = None,
        secret: bytes | None = None,
    ) -> None: ...
    def derive(self, key_material: bytes) -> bytes: ...
    def derive_into(self, key_material: bytes, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...
    def derive_phc_encoded(self, key_material: bytes) -> str: ...
    @classmethod
    def verify_phc_encoded(
        cls, key_material: bytes, phc_encoded: str, secret: bytes | None = None
    ) -> None: ...

class Argon2id:
    def __init__(
        self,
        *,
        salt: bytes,
        length: int,
        iterations: int,
        lanes: int,
        memory_cost: int,
        ad: bytes | None = None,
        secret: bytes | None = None,
    ) -> None: ...
    def derive(self, key_material: bytes) -> bytes: ...
    def derive_into(self, key_material: bytes, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...
    def derive_phc_encoded(self, key_material: bytes) -> str: ...
    @classmethod
    def verify_phc_encoded(
        cls, key_material: bytes, phc_encoded: str, secret: bytes | None = None
    ) -> None: ...

class HKDF:
    def __init__(
        self,
        algorithm: HashAlgorithm,
        length: int,
        salt: bytes | None,
        info: bytes | None,
        backend: typing.Any = None,
    ): ...
    @staticmethod
    def extract(
        algorithm: HashAlgorithm, salt: bytes | None, key_material: Buffer
    ) -> bytes: ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class HKDFExpand:
    def __init__(
        self,
        algorithm: HashAlgorithm,
        length: int,
        info: bytes | None,
        backend: typing.Any = None,
    ): ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class X963KDF:
    def __init__(
        self,
        algorithm: HashAlgorithm,
        length: int,
        sharedinfo: bytes | None,
        backend: typing.Any = None,
    ) -> None: ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class ConcatKDFHash:
    def __init__(
        self,
        algorithm: HashAlgorithm,
        length: int,
        otherinfo: bytes | None,
        backend: typing.Any = None,
    ) -> None: ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class ConcatKDFHMAC:
    def __init__(
        self,
        algorithm: HashAlgorithm,
        length: int,
        salt: bytes | None,
        otherinfo: bytes | None,
        backend: typing.Any = None,
    ) -> None: ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class KBKDFHMAC:
    def __init__(
        self,
        algorithm: HashAlgorithm,
        mode: Mode,
        length: int,
        rlen: int,
        llen: int | None,
        location: CounterLocation,
        label: bytes | None,
        context: bytes | None,
        fixed: bytes | None,
        backend: typing.Any = None,
        *,
        break_location: int | None = None,
    ) -> None: ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...

class KBKDFCMAC:
    def __init__(
        self,
        algorithm: typing.Any,
        mode: Mode,
        length: int,
        rlen: int,
        llen: int | None,
        location: CounterLocation,
        label: bytes | None,
        context: bytes | None,
        fixed: bytes | None,
        backend: typing.Any = None,
        *,
        break_location: int | None = None,
    ) -> None: ...
    def derive(self, key_material: Buffer) -> bytes: ...
    def derive_into(self, key_material: Buffer, buffer: Buffer) -> int: ...
    def verify(self, key_material: bytes, expected_key: bytes) -> None: ...
